From: npiggin@suse.de
To: Andrew Morton <akpm@linux-foundation.org>
Cc: linux-fsdevel@vger.kernel.org,
Mark Fasheh <mark.fasheh@oracle.com>,
linux-ext4@vger.kernel.org, Badari Pulavarty <pbadari@us.ibm.com>
Subject: [patch 17/41] ext3 convert to new aops.
Date: Fri, 25 May 2007 22:22:01 +1000 [thread overview]
Message-ID: <20070524053155.833877000@linux.local0.net> (raw)
In-Reply-To: 20070524052844.860329000@suse.de
[-- Attachment #1: fs-ext3-aops.patch --]
[-- Type: text/plain, Size: 8747 bytes --]
Cc: linux-ext4@vger.kernel.org
Cc: Linux Filesystems <linux-fsdevel@vger.kernel.org>
Signed-off-by: Nick Piggin <npiggin@suse.de>
Various fixes and improvements
Signed-off-by: Badari Pulavarty <pbadari@us.ibm.com>
fs/ext3/inode.c | 136 ++++++++++++++++++++++++++++++++++++--------------------
1 file changed, 88 insertions(+), 48 deletions(-)
Index: linux-2.6/fs/ext3/inode.c
===================================================================
--- linux-2.6.orig/fs/ext3/inode.c
+++ linux-2.6/fs/ext3/inode.c
@@ -1147,51 +1147,68 @@ static int do_journal_get_write_access(h
return ext3_journal_get_write_access(handle, bh);
}
-static int ext3_prepare_write(struct file *file, struct page *page,
- unsigned from, unsigned to)
+static int ext3_write_begin(struct file *file, struct address_space *mapping,
+ loff_t pos, unsigned len, unsigned flags,
+ struct page **pagep, void **fsdata)
{
- struct inode *inode = page->mapping->host;
+ struct inode *inode = mapping->host;
int ret, needed_blocks = ext3_writepage_trans_blocks(inode);
handle_t *handle;
int retries = 0;
+ struct page *page;
+ pgoff_t index;
+ unsigned from, to;
+
+ index = pos >> PAGE_CACHE_SHIFT;
+ from = pos & (PAGE_CACHE_SIZE - 1);
+ to = from + len;
retry:
+ page = __grab_cache_page(mapping, index);
+ if (!page)
+ return -ENOMEM;
+ *pagep = page;
+
handle = ext3_journal_start(inode, needed_blocks);
if (IS_ERR(handle)) {
+ unlock_page(page);
+ page_cache_release(page);
ret = PTR_ERR(handle);
goto out;
}
- if (test_opt(inode->i_sb, NOBH) && ext3_should_writeback_data(inode))
- ret = nobh_prepare_write(page, from, to, ext3_get_block);
- else
- ret = block_prepare_write(page, from, to, ext3_get_block);
+ ret = block_write_begin(file, mapping, pos, len, flags, pagep, fsdata,
+ ext3_get_block);
if (ret)
- goto prepare_write_failed;
+ goto write_begin_failed;
if (ext3_should_journal_data(inode)) {
ret = walk_page_buffers(handle, page_buffers(page),
from, to, NULL, do_journal_get_write_access);
}
-prepare_write_failed:
- if (ret)
+write_begin_failed:
+ if (ret) {
ext3_journal_stop(handle);
+ unlock_page(page);
+ page_cache_release(page);
+ }
if (ret == -ENOSPC && ext3_should_retry_alloc(inode->i_sb, &retries))
goto retry;
out:
return ret;
}
+
int ext3_journal_dirty_data(handle_t *handle, struct buffer_head *bh)
{
int err = journal_dirty_data(handle, bh);
if (err)
ext3_journal_abort_handle(__FUNCTION__, __FUNCTION__,
- bh, handle,err);
+ bh, handle, err);
return err;
}
-/* For commit_write() in data=journal mode */
-static int commit_write_fn(handle_t *handle, struct buffer_head *bh)
+/* For write_end() in data=journal mode */
+static int write_end_fn(handle_t *handle, struct buffer_head *bh)
{
if (!buffer_mapped(bh) || buffer_freed(bh))
return 0;
@@ -1206,78 +1223,100 @@ static int commit_write_fn(handle_t *han
* ext3 never places buffers on inode->i_mapping->private_list. metadata
* buffers are managed internally.
*/
-static int ext3_ordered_commit_write(struct file *file, struct page *page,
- unsigned from, unsigned to)
+static int ext3_ordered_write_end(struct file *file,
+ struct address_space *mapping,
+ loff_t pos, unsigned len, unsigned copied,
+ struct page *page, void *fsdata)
{
handle_t *handle = ext3_journal_current_handle();
- struct inode *inode = page->mapping->host;
+ struct inode *inode = file->f_mapping->host;
+ unsigned from, to;
int ret = 0, ret2;
+ from = pos & (PAGE_CACHE_SIZE - 1);
+ to = from + len;
+
ret = walk_page_buffers(handle, page_buffers(page),
from, to, NULL, ext3_journal_dirty_data);
if (ret == 0) {
/*
* generic_commit_write() will run mark_inode_dirty() if i_size
* changes. So let's piggyback the i_disksize mark_inode_dirty
* into that.
*/
loff_t new_i_size;
- new_i_size = ((loff_t)page->index << PAGE_CACHE_SHIFT) + to;
+ new_i_size = pos + copied;
if (new_i_size > EXT3_I(inode)->i_disksize)
EXT3_I(inode)->i_disksize = new_i_size;
- ret = generic_commit_write(file, page, from, to);
+ copied = generic_write_end(file, mapping, pos, len, copied,
+ page, fsdata);
+ if (copied < 0)
+ ret = copied;
+ } else {
+ unlock_page(page);
+ page_cache_release(page);
}
ret2 = ext3_journal_stop(handle);
if (!ret)
ret = ret2;
- return ret;
+ return ret ? ret : copied;
}
-static int ext3_writeback_commit_write(struct file *file, struct page *page,
- unsigned from, unsigned to)
+static int ext3_writeback_write_end(struct file *file,
+ struct address_space *mapping,
+ loff_t pos, unsigned len, unsigned copied,
+ struct page *page, void *fsdata)
{
handle_t *handle = ext3_journal_current_handle();
- struct inode *inode = page->mapping->host;
+ struct inode *inode = file->f_mapping->host;
int ret = 0, ret2;
loff_t new_i_size;
- new_i_size = ((loff_t)page->index << PAGE_CACHE_SHIFT) + to;
+ new_i_size = pos + copied;
if (new_i_size > EXT3_I(inode)->i_disksize)
EXT3_I(inode)->i_disksize = new_i_size;
- if (test_opt(inode->i_sb, NOBH) && ext3_should_writeback_data(inode))
- ret = nobh_commit_write(file, page, from, to);
- else
- ret = generic_commit_write(file, page, from, to);
+ copied = generic_write_end(file, mapping, pos, len, copied,
+ page, fsdata);
+ if (copied < 0)
+ ret = copied;
ret2 = ext3_journal_stop(handle);
if (!ret)
ret = ret2;
- return ret;
+ return ret ? ret : copied;
}
-static int ext3_journalled_commit_write(struct file *file,
- struct page *page, unsigned from, unsigned to)
+static int ext3_journalled_write_end(struct file *file,
+ struct address_space *mapping,
+ loff_t pos, unsigned len, unsigned copied,
+ struct page *page, void *fsdata)
{
handle_t *handle = ext3_journal_current_handle();
- struct inode *inode = page->mapping->host;
+ struct inode *inode = mapping->host;
int ret = 0, ret2;
int partial = 0;
- loff_t pos;
+ unsigned from, to;
- /*
- * Here we duplicate the generic_commit_write() functionality
- */
- pos = ((loff_t)page->index << PAGE_CACHE_SHIFT) + to;
+ from = pos & (PAGE_CACHE_SIZE - 1);
+ to = from + len;
+
+ if (copied < len) {
+ if (!PageUptodate(page))
+ copied = 0;
+ page_zero_new_buffers(page, from+copied, to);
+ }
ret = walk_page_buffers(handle, page_buffers(page), from,
- to, &partial, commit_write_fn);
+ to, &partial, write_end_fn);
if (!partial)
SetPageUptodate(page);
- if (pos > inode->i_size)
- i_size_write(inode, pos);
+ unlock_page(page);
+ page_cache_release(page);
+ if (pos+copied > inode->i_size)
+ i_size_write(inode, pos+copied);
EXT3_I(inode)->i_state |= EXT3_STATE_JDATA;
if (inode->i_size > EXT3_I(inode)->i_disksize) {
EXT3_I(inode)->i_disksize = inode->i_size;
@@ -1285,10 +1324,11 @@ static int ext3_journalled_commit_write(
if (!ret)
ret = ret2;
}
+
ret2 = ext3_journal_stop(handle);
if (!ret)
ret = ret2;
- return ret;
+ return ret ? ret : copied;
}
/*
@@ -1546,7 +1586,7 @@ static int ext3_journalled_writepage(str
PAGE_CACHE_SIZE, NULL, do_journal_get_write_access);
err = walk_page_buffers(handle, page_buffers(page), 0,
- PAGE_CACHE_SIZE, NULL, commit_write_fn);
+ PAGE_CACHE_SIZE, NULL, write_end_fn);
if (ret == 0)
ret = err;
EXT3_I(inode)->i_state |= EXT3_STATE_JDATA;
@@ -1706,8 +1746,8 @@ static const struct address_space_operat
.readpages = ext3_readpages,
.writepage = ext3_ordered_writepage,
.sync_page = block_sync_page,
- .prepare_write = ext3_prepare_write,
- .commit_write = ext3_ordered_commit_write,
+ .write_begin = ext3_write_begin,
+ .write_end = ext3_ordered_write_end,
.bmap = ext3_bmap,
.invalidatepage = ext3_invalidatepage,
.releasepage = ext3_releasepage,
@@ -1720,8 +1760,8 @@ static const struct address_space_operat
.readpages = ext3_readpages,
.writepage = ext3_writeback_writepage,
.sync_page = block_sync_page,
- .prepare_write = ext3_prepare_write,
- .commit_write = ext3_writeback_commit_write,
+ .write_begin = ext3_write_begin,
+ .write_end = ext3_writeback_write_end,
.bmap = ext3_bmap,
.invalidatepage = ext3_invalidatepage,
.releasepage = ext3_releasepage,
@@ -1734,8 +1774,8 @@ static const struct address_space_operat
.readpages = ext3_readpages,
.writepage = ext3_journalled_writepage,
.sync_page = block_sync_page,
- .prepare_write = ext3_prepare_write,
- .commit_write = ext3_journalled_commit_write,
+ .write_begin = ext3_write_begin,
+ .write_end = ext3_journalled_write_end,
.set_page_dirty = ext3_journalled_set_page_dirty,
.bmap = ext3_bmap,
.invalidatepage = ext3_invalidatepage,
@@ -1206,78 +1223,100 @@ static int commit_write_fn(handle_t *han
if (ret == 0) {
/*
- * generic_commit_write() will run mark_inode_dirty() if i_size
+ * generic_write_end() will run mark_inode_dirty() if i_size
* changes. So let's piggyback the i_disksize mark_inode_dirty
* into that.
--
next prev parent reply other threads:[~2007-05-25 12:37 UTC|newest]
Thread overview: 76+ messages / expand[flat|nested] mbox.gz Atom feed top
2007-05-25 12:21 [patch 00/41] Buffered write deadlock fix and new aops for 2.6.22-rc2-mm1 npiggin
2007-05-25 12:21 ` [patch 01/41] mm: revert KERNEL_DS buffered write optimisation npiggin
2007-05-25 12:21 ` npiggin
2007-05-25 12:21 ` [patch 02/41] Revert 81b0c8713385ce1b1b9058e916edcf9561ad76d6 npiggin
2007-05-25 12:21 ` npiggin, Andrew Morton
2007-05-25 12:21 ` [patch 03/41] Revert 6527c2bdf1f833cc18e8f42bd97973d583e4aa83 npiggin
2007-05-25 12:21 ` npiggin, Andrew Morton
2007-05-25 12:21 ` [patch 04/41] mm: clean up buffered write code npiggin
2007-05-25 12:21 ` npiggin, Andrew Morton
2007-05-25 12:21 ` [patch 05/41] mm: debug write deadlocks npiggin
2007-05-25 12:21 ` npiggin
2007-05-25 12:21 ` [patch 06/41] mm: trim more holes npiggin
2007-05-25 12:21 ` npiggin
2007-05-25 12:21 ` [patch 07/41] mm: buffered write cleanup npiggin
2007-05-25 12:21 ` npiggin
2007-05-25 12:21 ` [patch 08/41] mm: write iovec cleanup npiggin
2007-05-25 12:21 ` npiggin
2007-05-25 12:21 ` [patch 09/41] mm: fix pagecache write deadlocks npiggin
2007-05-25 12:21 ` npiggin
2007-05-25 12:21 ` [patch 10/41] mm: buffered write iterator npiggin
2007-05-25 12:21 ` npiggin
2007-05-25 12:21 ` [patch 11/41] fs: fix data-loss on error npiggin
2007-05-25 12:21 ` npiggin
2007-05-25 12:21 ` [patch 12/41] fs: introduce write_begin, write_end, and perform_write aops npiggin
2007-05-25 12:21 ` npiggin
2007-05-31 4:30 ` Andrew Morton
2007-05-31 4:30 ` Andrew Morton
2007-05-31 4:43 ` Nick Piggin
2007-05-31 4:43 ` Nick Piggin
2007-05-31 4:52 ` Andrew Morton
2007-05-31 4:52 ` Andrew Morton
2007-05-31 4:57 ` Nick Piggin
2007-05-31 4:57 ` Nick Piggin
2007-05-31 5:11 ` Andrew Morton
2007-05-31 5:11 ` Andrew Morton
2007-05-31 5:15 ` Nick Piggin
2007-05-31 5:15 ` Nick Piggin
2007-05-31 7:05 ` Andrew Morton
2007-05-31 7:05 ` Andrew Morton
2007-06-01 1:18 ` Nick Piggin
2007-06-01 1:18 ` Nick Piggin
2007-05-25 12:21 ` [patch 13/41] mm: restore KERNEL_DS optimisations npiggin
2007-05-25 12:21 ` npiggin
2007-05-25 12:21 ` [patch 14/41] implement simple fs aops npiggin
2007-05-25 12:21 ` [patch 15/41] block_dev convert to new aops npiggin
2007-05-25 12:22 ` [patch 16/41] ext2 " npiggin
2007-05-25 12:22 ` npiggin [this message]
2007-05-25 12:22 ` [patch 18/41] ext4 " npiggin
2007-05-25 12:22 ` [patch 19/41] xfs " npiggin
2007-05-25 12:22 ` [patch 20/41] fs: new cont helpers npiggin
2007-05-25 12:22 ` [patch 21/41] fat convert to new aops npiggin
2007-05-25 12:22 ` [patch 22/41] adfs " npiggin
2007-05-26 9:14 ` Russell King
2007-05-27 1:35 ` Nick Piggin
2007-05-25 12:22 ` [patch 23/41] hfs " npiggin
2007-05-25 12:22 ` [patch 24/41] hfsplus " npiggin
2007-05-25 12:22 ` [patch 25/41] hpfs " npiggin
2007-05-25 12:22 ` [patch 26/41] bfs " npiggin
2007-05-25 12:22 ` [patch 27/41] qnx4 " npiggin
2007-05-25 14:35 ` Anders Larsen
2007-05-26 7:23 ` Nick Piggin
2007-05-25 12:22 ` [patch 28/41] reiserfs use generic write npiggin
2007-05-25 12:22 ` [patch 29/41] reiserfs convert to new aops npiggin
2007-05-25 12:22 ` [patch 30/41] reiserfs use generic_cont_expand_simple npiggin
2007-05-25 12:22 ` [patch 31/41] With reiserfs no longer using the weird generic_cont_expand, remove it completely npiggin
2007-05-25 12:22 ` [patch 32/41] nfs convert to new aops npiggin
2007-05-25 12:22 ` [patch 33/41] smb " npiggin
2007-05-25 12:22 ` [patch 34/41] fuse " npiggin
2007-05-25 12:22 ` [patch 35/41] hostfs " npiggin
2007-05-25 12:22 ` [patch 36/41] jffs2 " npiggin
2007-05-25 12:22 ` [patch 37/41] ufs " npiggin
2007-05-25 12:22 ` [patch 38/41] udf " npiggin
2007-05-25 12:22 ` [patch 39/41] sysv " npiggin
2007-05-25 12:22 ` [patch 40/41] minix " npiggin
2007-05-25 12:22 ` [patch 41/41] jfs " npiggin
2007-05-25 22:55 ` [patch 00/41] Buffered write deadlock fix and new aops for 2.6.22-rc2-mm1 Mark Fasheh
Reply instructions:
You may reply publicly to this message via plain-text email
using any one of the following methods:
* Save the following mbox file, import it into your mail client,
and reply-to-all from there: mbox
Avoid top-posting and favor interleaved quoting:
https://en.wikipedia.org/wiki/Posting_style#Interleaved_style
* Reply using the --to, --cc, and --in-reply-to
switches of git-send-email(1):
git send-email \
--in-reply-to=20070524053155.833877000@linux.local0.net \
--to=npiggin@suse.de \
--cc=akpm@linux-foundation.org \
--cc=linux-ext4@vger.kernel.org \
--cc=linux-fsdevel@vger.kernel.org \
--cc=mark.fasheh@oracle.com \
--cc=pbadari@us.ibm.com \
/path/to/YOUR_REPLY
https://kernel.org/pub/software/scm/git/docs/git-send-email.html
* If your mail client supports setting the In-Reply-To header
via mailto: links, try the mailto: link
Be sure your reply has a Subject: header at the top and a blank line
before the message body.
This is an external index of several public inboxes,
see mirroring instructions on how to clone and mirror
all data and code used by this external index.